from feds_shield.decrypt_env import *

import unittest

'''
python3 -m unittest test_decrypt_env
'''
class Testing(unittest.TestCase):

    def test__DecryptEnv1__decode(self):
        s = "nu_shield1"
        m = "wf1.txt"
        b = ("bk10","bk11")
        a = "std_alpha"
        de = DecryptEnv1(s,m,b,a)
        de.decode()
        assert de.deco == "WHERE is HERE"

if __name__ == '__main__':
    unittest.main()



import csv

def modifini(filename): 
    assert "." not in filename
    return "data/" + filename + ".csv"

def save_shield_to_file(shield,filename):
    fp = modifini(filename)

    with open(fp, 'w') as fi:
        wtr = csv.writer(fi,delimiter=',',lineterminator='\n')
        for s in shield:
            wtr.writerow(s)
    return

def load_shield_from_file(f):
    fp = modifini(f)
    q = []
    with open(fp, 'r') as file:
        reader = csv.reader(file)
        for row in reader:
            q.append([int(r) for r in row])
    return q


#    def __init__(self,shield,fk,fek):

from feds_shield.structure import *
from feds_shield.bi_key import *
import unittest


'''
python3 -m unittest test_structure
'''
class Testing(unittest.TestCase):

    def test__Structure__register_fire_on_layer(self):
        
        # case 1
        shield = load_shield_from_file('x')
        fk = load_BiKey("bk10","bk11")
        fek = load_BiKey("bk20","bk21")
        s = Structure(zero_shield(shield),fk,fek)
        s.register_fire_on_layer(s.fk,97,0,False)
        assert unpair_shield(s.shield) == [[1, 3, 4], [5, 10, 20], [10, 20], [20, 30]]

        # case 2
        fk2 = load_BiKey("bk30","bk31")
        s = Structure(zero_shield(shield),fk2,fek)
        s.register_fire_on_layer(s.fk,97,0,False)
        assert unpair_shield(s.shield) == [[0, 22, 13], [5, 10, 20], [10, 20], [20, 30]]
        return

if __name__ == '__main__':
    unittest.main()



from feds_shield.file_ops import *
import unittest

'''
python3 -m unittest test_file_ops
'''
class Testing(unittest.TestCase):
        
    def test__save_shield_to_file(self):
        s = [[1,3,4],[5,10,20],[10,20],[20,30]]
        f = "x"
        save_shield_to_file(s,f)

    def test__load_shield_from_file(self):
        f = "x"
        q = load_shield_from_file(f)
        assert q == [[1,3,4],[5,10,20],[10,20],[20,30]]

if __name__ == '__main__':
    unittest.main()



'''
performs a statistical test to determine
the degree of bijectivity of the bi_key
given the alphabet. 

provided are basic tests and are non-conclusive
for determining whether a key satisfies cryptographic
standards.
'''
from .bi_key import *

######################################################

'''
constructs a mapping a -> bk.apply_int(i,a); a in intAlphabet
'''
def bijective_dictionary__BiKey_on_int(bk,i,intAlphabet):
    
    d = {}

    for j in intAlphabet:
        j2 = bk.apply_int(i,j)[0]
        d[j] = j2
    return d

'''
NOTE: does not take into account the confines of SPACE(bk.apply_int(*))
      w.r.t. intAlphabet.
'''
def bijective_measure_on_dictionary(d):
    return len(set(d.values())) / len(d)
    
'''
calculates the mean bijective measure of BiKey
on the range of alphabetic characters

bk := key
intAlphabet := list(int)
'''
def bijective_measure_test__BiKey(bk,intAlphabet):
    m = 0.0
    for a in intAlphabet:
        m += bijective_measure_on_dictionary(bijective_dictionary__BiKey_on_int(bk,a,intAlphabet))
    return m / len(intAlphabet)


#############################################################

def identity_measure_on_dictionary(d):
    if len(d) == 0: return 0.
    c = 0
    for (k,v) in d.items():
        if k == v:
            c += 1
    return c / len(d)

'''
calculates the mean identity measure of BiKey
on the range of alphabetic characters

bk := key
intAlphabet := list(int)
'''
def identity_measure_test__BiKey(bk,intAlphabet):
    m = 0.0
    for a in intAlphabet:
        m += identity_measure_on_dictionary(bijective_dictionary__BiKey_on_int(bk,a,intAlphabet))
    return m / len(intAlphabet)

#############################################################

# TODO: def random_shield_values(layerSizes)

# test bk_case1
DEFAULT_TEST_ALPHABET = list(range(0,100))

def bijective_measure_test__case1(f1,f2,intAlphabet = DEFAULT_TEST_ALPHABET):
    intAlphabet = list(range(0,100))
    bk = load_BiKey(f1,f2)
    bk.modulo = 100
    return bijective_measure_test__BiKey(bk,intAlphabet)

def identity_measure_test__case1(f1,f2,intAlphabet = DEFAULT_TEST_ALPHABET):
    intAlphabet = list(range(0,100))
    bk = load_BiKey(f1,f2)
    bk.modulo = 100
    return identity_measure_test__BiKey(bk,intAlphabet)

"""
bijective_measure_test__case1("bk10","bk11")
identity_measure_test__case1("bk10","bk11")
"""



'''
loads user-file alphabet into memory  
'''
import json
from .file_ops import *
    
def save_alphabet(d,f):
    with open(modifini(f),'w') as fi:
        return json.dump(d,fi)

def load_alphabet(f):
    with open(modifini(f),'r') as fi:
        return json.load(fi)


std_symbols = ['\n','@','.','!','?','(',')','[',']','{','}','$','%','\t', ' ']

'''
saves standard alphabet:
a-zA-Z0-9 
.!?()[]{}$%

into file `std_alpha.csv`
'''
def save_std_alphabet():
    d = {}

    # 97-123
    for i in range(97,123):
        d[chr(i)] = i
    # 65-91
    for i in range(65,91):
        d[chr(i)] = i
    # 0-9
    for i in range(48,58):
        d[chr(i)] = i

    # 33,63,96
    for s in std_symbols:
        d[s] = ord(s) 

    save_alphabet(d,'std_alpha')


"""
friends are odd,enemies are even
"""
def shield_idn_enemy(shield):
    s2 = []
    for (i,s) in enumerate(shield):
        for (j,s_) in enumerate(s):
            if not s_ % 2:
                s2.append(s_ + i)
    return s2 

class Centerion: 

    """
    feForm := list<enemy cell idn + its index>
    """
    def __init__(self):
        # friend/enemy formation
        self.feForm = None
        return

    def load_fe(self,shield):
        self.feForm = shield_idn_enemy(shield)




from .bi_key import *
from .alphabet_load import * 
from .structure import * 
from copy import deepcopy

#WARNING: alphabet assumed to be 1-to-1
"""
"""
def invert_alphabet(a):
    a2 = {}
    for k,v in a.items():
        a2[v] = k
    return a2 

"""
a basic decryption environment class
decrypts character by character
"""
class DecryptEnv1:

    def __init__(self,shieldFile,msgFile,biKeyFile,alphabetFile): 
        self.shieldFile = shieldFile
        self.shieldObj = load_shield_from_file(self.shieldFile)
        self.shieldLen = sum([len(x) for x in self.shieldObj]) * 2
        self.shieldHole = int(len(self.shieldObj[0]) / 2) 
        self.msgFile = msgFile        

        # open message file in reverse
        with open("data/" + self.msgFile,"r") as mf: 
            msg = mf.readlines()
            msg = msg[0][:-1]
            msg = msg.split(",")
            self.msg = [int(x) for x in msg][::-1]

        self.bk = load_BiKey(biKeyFile[0],biKeyFile[1])
        self.alphabet = invert_alphabet(load_alphabet(alphabetFile))
        self.deco = ""
        self.decoh = [] 

        # each is char encoding
        # s2 s1
        self.s1,self.s2 = None,None
        self.is_active = True

    def decode(self):
        while self.decode_one():continue

    """
    """
    def decode_one(self):

        if not self.is_active:
            print("[!!] BROKEN ENCRYPTION")
            return False 
        
        if len(self.msg) == 0:
            # case: use initial shield
            self.s1 = deepcopy(self.s2)
            self.s2 = zero_shield(self.shieldObj)
            s2 = []
            for q in self.s2: s2.extend(q)
            self.s2 = s2

            qi = self.decode_pair()
            self.register_decode_pair(qi)
            if not self.is_active:
                print("[!!] BROKEN ENCRYPTION")
                return False
            self.deco += self.alphabet[qi]

            ##3 ?? 
            self.deco = self.deco[::-1]
            return False

        l = self.msg.pop(0)
        self.pop_pad(l)
        x = self.shieldLen
        
        # fetch the msg
        q = []
        for i in range(0,self.shieldLen,2):
            q.append((self.msg[1],self.msg[0]))
            self.msg.pop(0)
            self.msg.pop(0)
        q = q[::-1]

        # case: initial, re-loop to collect parent
        if type(self.s2) == type(None):
            self.s2 = q
            return self.decode_one()

        # decode
        self.s1 = deepcopy(self.s2)
        self.s2 = q
        qi = self.decode_pair()
        self.register_decode_pair(qi)
        if not self.is_active:
            print("[!!] BROKEN ENCRYPTION")
            return False
        self.deco += self.alphabet[qi]
        self.decoh.append(self.s1)
        return True 

    def decode_pair(self):
        # decode pair
        i = self.shieldHole
        try:
            return self.bk.invert_apply_int(self.s2[i][0],self.s1[i][0],self.s1[i][1])
        except:
            return False

    def register_decode_pair(self,dp): 
        if type(dp) == bool:
            self.is_active = False
        if dp not in self.alphabet:
            self.is_active = False 
        return

    def pop_pad(self,x):
        while x > 0:
            self.msg = self.msg[1:]
            x -= 1



from .structure import *
from .bi_key import * 
from .file_ops import *
from .firing_agent import *
from copy import deepcopy
import csv

"""
a basic encryption environment class
"""
class CryptEnv1: 

    def __init__(self,alphabetFile,shieldFile,bk1File,bk2File,msg,writeFile):
        self.fa = FiringAgent(msg,alphabetFile)
        shield = zero_shield(load_shield_from_file(shieldFile))

        bk1 = load_BiKey(bk1File[0],bk1File[1])
        bk2 = load_BiKey(bk2File[0],bk2File[1])
        self.structure = Structure(shield,bk1,bk2)

        self.initialized = False
        self.wf = "data/" + writeFile
        self.wfo = open(self.wf,"w")
        self.cw = csv.writer(self.wfo,delimiter=",")
        ##self.q = []

    """
    return:
    """
    def encode_one(self):
        x = self.fa.fire()
        print("encoding ",x)

        if type(x) == type(None):
            self.wfo.close()
            return False
        self.structure.absorb_fire(x)
        self.write_to_file()
        self.wfo.write(",")
        return True
        
    def write_to_file(self):
        #shield = unpair_shield(deepcopy(self.structure.shield))
        s2 = []
        for s in self.structure.shield:
            sx = []
            for s_ in s: sx.extend(s_)
            s2.extend([str(s_) for s_ in sx])
        s2 = ",".join(s2)
        self.wfo.write(s2)
        #
        ##self.q.append(s2)
        self.wfo.write(",")
        self.paddy_pat()
        return

    """
    what is a friend or enemy without recognition?
    and however can these terms can figured without
    a center (Centerion)?
    """
    def paddy_pat(self):
        l = len(self.structure.c.feForm)
        sef = [str(s) for s in self.structure.c.feForm]
        sef.append(str(l))
        self.wfo.write(",".join(sef))

def test__CryptEnv1__encode_one():

    a = "std_alpha"
    s = "nu_shield1"
    bk1 = ("bk10","bk11")
    bk2 = ("bk20","bk21")
    msg = "WHERE is HERE"
    writeFile = "data/wf1.txt"
    ce = CryptEnv1(a,s,bk1,bk2,msg,writeFile)

    while ce.encode_one():
        continue

    # test 
    return ce


from feds_shield.bi_key import *
import unittest

'''
python3 -m unittest test_bi_key
'''
class Testing(unittest.TestCase):
        
    def test__BiKey__make_key1(self):
        bc = bk_case1()
        bk = BiKey(bc[0],bc[1],bc[2])
        bk.make_key()
        q = [c[0] for c in bc[1]]
        for i,q_ in enumerate(q): 
            assert bk.kd[q_][0] == bc[0][i]
            assert bk.kd[q_][1] == bc[1][i][1]
        return

    def test__BiKey__make_key2(self):
        bc = bk_case2()
        bk = BiKey(bc[0],bc[1],bc[2])
        bk.make_key()
        q = [(x1,x2[1]) for (x1,x2) in zip(bc[0],bc[1])]
        assert bk.kd == q
        return

    def test__BiKey__make_key3(self):
        bc = bk_case3()
        bk = BiKey(bc[0],bc[1],bc[2])
        bk.make_key()
        q = [c[0] for c in bc[1]]
        for i,q_ in enumerate(q): 
            assert bk.kd[q_][0] == bc[0][i]
            assert bk.kd[q_][1] == bc[1][i][1]
        return

    def test__BiKey__make_key4(self):
        bc = bk_case4()
        bk = BiKey(bc[0],bc[1],bc[2])
        bk.make_key()
        assert len(bk.kd) == 8

    """
    demonstrates using bi-key
    """
    def test__BiKey__apply_int__case1(self):
        bc = bk_case1()
        bk = BiKey(bc[0],bc[1],bc[2])
        bk.make_key()
        q = bk.apply_int(20,98)[0]
        assert q in [0,18]

    def test__BiKey__reapply_int__case1(self):
        bk = load_BiKey("bk10","bk11")
        q = bk.apply_int(20,98)[0] 
        assert q == 18

    def test__BiKey__invert_apply_int(self):
        bk = load_BiKey("bk10","bk11")

        f = 39
        t = 61
        t2 = bk.apply_int(f,t)
        tx = bk.invert_apply_int(t,t2[0],t2[1])
        assert(tx == f)

        f= 65
        t = 23
        t2 = bk.apply_int(f,t)
        tx = bk.invert_apply_int(t,t2[0],t2[1])
        assert(tx == f)
            



if __name__ == '__main__':
    unittest.main()
